//+---------------------------------------------------------------------------------------------------------------------+
//|                                                                                            ___fractal_dimension.mq4 |
//|                                                                                         iliko [arcsin5@netscape.net]|
//|                                                                                                                     |
//|                                                                                                                     |
//|  The Fractal Dimension Index determines the amount of market volatility. The easiest way to use this indicator is   |
//|  to understand that a value of 1.5 suggests the market is acting in a completely random fashion.                    |
//|  As the market deviates from 1.5, the opportunity for earning profits is increased in proportion                    |
//|  to the amount of deviation.                                                                                        |
//|  But be carreful, the indicator does not show the direction of trends !!                                            |
//|                                                                                                                     |
//|  The indicator is RED when the market is in a trend. And it is blue when there is a high volatility.                |
//|  When the FDI changes its color from red to blue, it means that a trend is finishing, the market becomes            |
//|  erratic and a high volatility is present. Usually, these "blue times" do not go for a long time. They come before  |
//|  a new trend.                                                                                                       |
//|                                                                                                                     |
//|  For more informations, see                                                                                         |
//|  http://www.forex-tsd.com/suggestions-trading-systems/6119-tasc-03-07-fractal-dimension-index.html                  |
//|                                                                                                                     |
//|                                                                                                                     |
//|  HOW TO USE INPUT PARAMETERS :                                                                                      |
//|  -----------------------------                                                                                      |
//|                                                                                                                     |
//|      1) e_period [ integer >= 1 ]                                              =>  30                               |
//|                                                                                                                     |
//|         The indicator will compute the historical market volatility over this period.                               |
//|         Choose its value according to the average of trend lengths.                                                 |
//|                                                                                                                     |
//|      2) e_type_data [ int = {PRICE_CLOSE = 0,                                                                       |
//|                              PRICE_OPEN  = 1,                                                                       |
//|                              PRICE_HIGH  = 2,                                                                       |
//|                              PRICE_LOW   = 3,                                                                       |
//|                              PRICE_MEDIAN    (high+low)/2              = 4,                                         |
//|                              PRICE_TYPICAL   (high+low+close)/3        = 5,                                         |
//|                              PRICE_WEIGHTED  (high+low+close+close)/4  = 6}     => PRICE_CLOSE                      |
//|                                                                                                                     |
//|         Defines on which price type the Fractal Dimension is computed.                                              |
//|                                                                                                                     |
//|      3) e_random_line [ 0.0 < double < 2.0 ]                                   => 1.5                               |
//|                                                                                                                     |
//|         Defines your separation betwen a trend market (red) and an erratic/high volatily one.                       |
//|                                                                                                                     |
//|                                                                                                                     |
//|                                                                                                                     |
//|                                                                                                                     |
//|  v1.0 - February 2007                                                                                               |
//+---------------------------------------------------------------------------------------------------------------------+
#property link      "arcsin5@netscape.net"

#property indicator_separate_window

#property indicator_levelcolor LimeGreen
#property indicator_levelwidth 1
#property indicator_levelstyle STYLE_DASH

#property indicator_buffers 2
#property indicator_color1 Blue
#property indicator_color2 Red
#property indicator_width1 2
#property indicator_width2 2
#property strict

//************************************************************
// Includes
//************************************************************



//************************************************************
// Input parameters
//************************************************************

extern ENUM_TIMEFRAMES    TimeFrame                = PERIOD_CURRENT; // Time frame to use
extern int                e_period                 = 100;            // Calculation period
extern ENUM_APPLIED_PRICE e_type_data              = PRICE_CLOSE;    // Price to use
extern double             e_random_line            = 0.5;            // Random line at : 
extern ENUM_MA_METHOD     e_price_filtering_mode   = MODE_SMA;       // Average method
extern int                e_price_filtering_period = 1;              // Price filtering period



//************************************************************
// Constant
//************************************************************
string INDICATOR_NAME = "Hurst Exponent";
string FILENAME       = "___fractal_dimension.mq4";

double LOG_2;



//************************************************************
// Private vars
//************************************************************

//---- input buffer
double ExtInputBuffer[];

//---- output buffer
double ExtOutputBufferUp[];
double ExtOutputBufferDown[];


int g_period_minus_1;
string indicatorFileName;
bool   returnBars;



//+================================================================================================================+
//+=== FUNCTION : init                                                                                          ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== Initialization function                                                                                  ===+
//+=== Check the user input parameters and convert them in appropriate types.                                   ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int init()
{ 
   // Check e_period input parameter
   if( e_period < 2 )
   {
      Alert( "[ 10-ERROR  " + FILENAME + " ] input parameter \"e_period\" must be >= 1 (" + (string)e_period + ")" );
      return( -1 );
   }
   if( e_type_data < PRICE_CLOSE || e_type_data > PRICE_WEIGHTED )
   {
      Alert( "[ 20-ERROR  " + FILENAME + " ] input parameter \"e_type_data\" unknown (" + (string)e_type_data + ")" );
      return( -1 );
   }
   if( e_random_line < 0.0 || e_random_line > 2.0 )
   {
      Alert( "[ 30-ERROR  " + FILENAME + " ] input parameter \"e_random_line\" = " + (string)e_random_line + " out of range ( 0.0 < e_random_line < 2.0 )" );
      return( -1 );
   }

   
   IndicatorBuffers( 3 );
   SetIndexBuffer( 0, ExtOutputBufferUp );
   SetIndexBuffer( 1, ExtOutputBufferDown );
   SetIndexBuffer( 2, ExtInputBuffer );
   
   SetLevelValue( 0, e_random_line );
   
   IndicatorShortName( INDICATOR_NAME );  
   SetIndexLabel( 0, INDICATOR_NAME );

   SetIndexDrawBegin( 0, 2 * e_period );
  
   g_period_minus_1 = e_period - 1;
   LOG_2 = MathLog( 2.0 );
         indicatorFileName = WindowExpertName();
         returnBars        = TimeFrame==-99;
         TimeFrame         = MathMax(TimeFrame,_Period);
   IndicatorShortName(timeFrameToString(TimeFrame)+" fractal dimension");
   return( 0 );
}
 
//+================================================================================================================+
//+=== FUNCTION : deinit                                                                                        ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== Custor indicator deinitialization function                                                               ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int deinit()
{
   return(0);
}
 
//+================================================================================================================+
//+=== FUNCTION : start                                                                                         ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== This callback is fired by metatrader for each tick                                                       ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+================================================================================================================+
int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
         int limit = MathMin(Bars-counted_bars,Bars-1);
         if (returnBars) { ExtOutputBufferUp[0] = MathMin(limit+1,Bars-1); return(0); }
         if (TimeFrame!=_Period)
         {
            limit = (int)MathMax(limit,MathMin(Bars-1,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
            for (int i=limit; i>=0; i--)
            {
               int y = iBarShift(NULL,TimeFrame,Time[i]);
               ExtOutputBufferUp[i]   = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,e_period,e_type_data,e_random_line,e_price_filtering_mode,e_price_filtering_period,0,y);
               ExtOutputBufferDown[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,e_period,e_type_data,e_random_line,e_price_filtering_mode,e_price_filtering_period,1,y);
            }
            return(0);
         }                        
         _computeFdi( limit );

   return( 0 );
}

//+================================================================================================================+
//+=== FUNCTION : _computeFdi                                                                                   ===+
//+===                                                                                                          ===+
//+===                                                                                                          ===+
//+=== Compute FDI values from input data.                                                                      ===+
//+===                                                                                                          ===+
//+=== In :                                                                                                     ===+
//+===    - lastBars : these "n" last bars must be repainted                                                    ===+
//+===                                                                                                          ===+
//+================================================================================================================+
void _computeFdi( int lastBars )
{
   int    pos, iteration;
   double diff, priorDiff;
   double length; 
   double priceMax, priceMin;
   double fdi;  
        

   for( pos = lastBars; pos >= 0; pos-- )
   {      
      ExtInputBuffer[pos] = iMA(NULL,0,e_price_filtering_period,0,e_price_filtering_mode,e_type_data,pos);
      priceMax = ExtInputBuffer[ArrayMaximum(ExtInputBuffer,e_period, pos)];
      priceMin = ExtInputBuffer[ArrayMinimum(ExtInputBuffer,e_period, pos)];
      
      length    = 0.0;
      priorDiff = 0.0;
      
      for( iteration = 0; iteration <= g_period_minus_1; iteration++ )
      {
         if( ( priceMax - priceMin ) > 0.0 )
         {
            diff = ( ExtInputBuffer[(int)MathMin(pos + iteration,Bars-1)] - priceMin ) / ( priceMax - priceMin );
            if( iteration > 0 )
            {
               length += MathSqrt( MathPow( diff - priorDiff, 2.0 ) + ( 1.0 / MathPow( e_period, 2.0 ) ) );
            }
            priorDiff = diff;
         }
      }
         
      if( length > 0.0 )
      {
         fdi = 1.0 + ( MathLog( length ) + LOG_2 ) / MathLog( 2 * e_period );
      }
      else
      {
         /*
         ** The FDI algorithm suggests in this case a zero value.
         ** I prefer to use the previous FDI value.
         */
         fdi = 0.0;
      } 
      if (pos<Bars-1)
      if( (2-fdi) > e_random_line )
      {
         ExtOutputBufferUp[pos]   = 2-fdi;
         ExtOutputBufferUp[pos+1] = MathMin( ExtOutputBufferUp[pos+1], ExtOutputBufferDown[pos+1] );
         ExtOutputBufferDown[pos] = EMPTY_VALUE;
      }
      else
      {
         ExtOutputBufferDown[pos]   = 2-fdi;
         ExtOutputBufferDown[pos+1] = MathMin( ExtOutputBufferUp[pos+1], ExtOutputBufferDown[pos+1] );
         ExtOutputBufferUp[pos] = EMPTY_VALUE;
      } 
   }

}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}